@@ -15,7 +15,7 @@ extern "C" {
   /* get the list element named str, or return NULL */
   /*This code comes from the R-exts documentation */
  
-  SEXP getListElement(SEXP list, char *str)
+  SEXP getListElement(SEXP list, const char *str)
   {
     SEXP elmt = R_NilValue, names = getAttrib(list, R_NamesSymbol);
     int i;
@@ -653,7 +653,7 @@ read in landscapes
 	
 	///Vital Vectors:  extinctions
 	SEXP Evec = PROTECT(allocVector(REALSXP, L.gethabs()));
-	double ev[L.gethabs()];
+	double *ev = new double[L.gethabs()];
 	L.getextinct(e,ev);
 	for (i=0;i<L.gethabs();i++)
 	  {
@@ -663,7 +663,7 @@ read in landscapes
 	
 	///Vital Vectors:  carry
 	SEXP Kvec = PROTECT(allocVector(REALSXP, L.gethabs()));
-	int cv[L.gethabs()];
+	int *cv = new int[L.gethabs()];
 	L.getk(e,cv);
 	for (i=0;i<L.gethabs();i++)
 	  {
@@ -675,7 +675,7 @@ read in landscapes
 	///demography in a habitat.  This vector is the length of the
 	///number of local demographies
 	
-	double dv[L.getndemo()];
+	double *dv = new double[L.getndemo()];
 	SEXP LDvec = PROTECT(allocVector(REALSXP, L.getndemo()));
 	L.getldemovector(e,dv);
 	for (i=0;i<L.getndemo();i++)
@@ -690,7 +690,10 @@ read in landscapes
 	SEXP  rightx = PROTECT(allocVector(REALSXP, L.gethabs()));
 	SEXP  topy   = PROTECT(allocVector(REALSXP, L.gethabs()));
 	SEXP  boty   = PROTECT(allocVector(REALSXP, L.gethabs()));
-	double lx[L.gethabs()], rx[L.gethabs()], tpy[L.gethabs()],bty[L.gethabs()];
+	double *lx = new double[L.gethabs()];
+	double *rx = new double[L.gethabs()];
+	double *tpy= new double[L.gethabs()];
+	double *bty= new double[L.gethabs()];
 	L.getpoploc(e,lx,rx,tpy,bty);
 	for (i=0;i<L.gethabs();i++)
 	  {
@@ -700,6 +703,11 @@ read in landscapes
 	    REAL(boty)[i]   = bty[i];
 	  }
 	
+	delete [] lx;
+	delete [] rx;
+	delete [] tpy;
+	delete [] bty;
+
 #ifdef RDEBUG
 	cerr <<"Finished setting up vectors for epoch: "<<e<<endl;
 #endif
